#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include <QDoubleSpinBox>
#include <QPushButton>
#include <QList>
#include <QHash>
#include <QTimer>

class SimulationMesh;

namespace Ui
{
    class MainWindow;
}


class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void stepPressed();
    void playPressed();
    void simTimedout();
    void editMeshPressed();

private:

    // UI-stash
    Ui::MainWindow *ui;

    QHash<int,QDoubleSpinBox*> tempDSpins;
    QHash<int,QDoubleSpinBox*> fuelDSpins;
    QList<QDoubleSpinBox*> distDSpins;
    QList<QDoubleSpinBox*> paramsDSpins;

    QPushButton* editMeshButton;
    QPushButton* stepButton;
    QPushButton* playButton;
    QDoubleSpinBox* speedBox;

    QTimer* stepSimTimer;

    // Other variables
    SimulationMesh* mesh;
    bool simRunning;
    bool editingMesh;

    // private helper methods:
    int getLinkfrom(int);
    int getLinkto(int);
    int getDistdspinIntProperty(int, const char*);
    int findMaxNbors(void);
    void getUiInputWidgets();
    void prepareSimulation();
    void stepSimulation();
    void lockOptions(bool);
    void lockMesh();
    void unlockMesh();
    void saveMeshValues();
    void updateDistColors();
    void updateFuelColors();
    void updateTempColors();
	


};

#endif // MAINWINDOW_H
