#pragma once
#include "RenderObject.h"

#define ARGNO_TEMPKERN_TEMPS 0
#define ARGNO_TEMPKERN_READIGN 1
#define ARGNO_TEMPKERN_WRITEIGN 2
#define ARGNO_TEMPKERN_FUEL 3
#define ARGNO_TEMPKERN_NBORSWIDTH 4
#define ARGNO_TEMPKERN_NBORS 5
#define ARGNO_TEMPKERN_DISTANCES 6
#define ARGNO_TEMPKERN_NBORCOUNTS 7
#define ARGNO_TEMPKERN_LINKSPEEDS 8
#define ARGNO_TEMPKERN_DELTAT 9

#define ARGNO_WINDKERN_LINKDIRS 0
#define ARGNO_WINDKERN_NBORSWIDTH 1
#define ARGNO_WINDKERN_WINDVEC 2
#define ARGNO_WINDKERN_LINKSPEEDS 3

#define ARGNO_COLKERN_READTEMPS 0
#define ARGNO_COLKERN_FUELINIT 1
#define ARGNO_COLKERN_FUELREM 2
#define ARGNO_COLKERN_MAPPING 3
#define ARGNO_COLKERN_COLORS 4
#define ARGNO_COLKERN_DISPL 5

// forward declarations:
class Mesh;
struct VertexDataPointers;

class BurningObject :
	public RenderObject
{
private:	
	cl_context clCtxt; //context
	cl_device_id* clDevices; //devices
	cl_command_queue clQueue;
	cl_program clProgram;
	cl_mem clTempsbuff;
#define NUM_IGNBUFFS 2
	cl_mem clIgnbuff[NUM_IGNBUFFS];//need two; one is updated from the other, then copied back.
	cl_mem clFuelsbuff;
	cl_mem clFuelremsbuff;
	cl_mem clNborsbuff;
	cl_mem clNborCountsbuff;
	cl_mem clDistbuff;
	cl_mem clLinkDirbuff;
	cl_mem clWindVecbuff;
	cl_mem clLinkSpeedbuff;
	cl_mem clMappingbuff;
#define NUM_SHAREDBUFFS 2
#define SHAREDBUFF_COLOR 0
#define SHAREDBUFF_DISPL 1
	cl_mem clSharedbuff[NUM_SHAREDBUFFS];// Buffers that are shared between gl and cl: color and displacement
	cl_kernel clTempKern;
	//cl_kernel clCopyKern;
	cl_kernel clColorKern;
	cl_kernel clWindKern;
	size_t clWorkSize[1];
	int numNodes;
	float windVec[3];

public:
	BurningObject(GLfloat* vertexPositions, GLfloat* vertexNormals, GLfloat* vertexColors, 
		GLfloat* vertexTexcoords, GLfloat* vertexDispl, GLint* facelookuptable, GLint numVertices, 
		GLint numDrawVerts, const char* vertShaderFilename, const char* fragShaderFilename, const char* textureFilename, Mesh* simMesh);
	~BurningObject(void);
	
	void initOpenCL(void);
	void makeLookupTables(const VertexDataPointers* rawVertexLookup);
	void makeNeighbourMatrix(void);
	void makeDistMatrix(void);
	void loadDataFromObj(const GLchar* objFileName, long maxFileLength, int maxLineLength, GLfloat scale);
	void setElementCountsFromObj(std::ifstream &instream, int maxLineLength);
	void initCLBuffers(Mesh* simMesh);
	void updateTemperatures(GLfloat deltaT);
	void updateWind(GLfloat wind_x, GLfloat wind_y, GLfloat wind_z);
};
