#pragma once

/*********************
 * Assumptions about the obj-file:
 * - vertex positions are unique (so I can create a mesh of unique nodes)
 * - all vertex positions are used in faces (so I know the size of the vertex table)?
 * - all faces made up of same nuber of vertices (must be three right now) (for loops and similar)
 *********************/

// quick fix for bug in jtreesimple_corr.obj - set to zero if not using jtreesimple_corr.
#define JORGENTREE_BUGFIX 0

// Pointers into the lists that holds the data for the vertices.
struct VertexDataPointers {
	//GLint positionIdx; this is the first index of the vector (and so does not need to be stored explicitly)
	GLint normalIdx;
	GLint texIdx;
	GLint colorIdx;
};

struct RawFaceDataPointers {
	GLint positionIdx; //this is the first index of the vector (and so does not need to be stored explicitly)
	GLint normalIdx;
	GLint texIdx;
	GLint colorIdx;
};

// Index into the vertices-table (that is grouped on position indexes)
struct VertexIndex {
	GLint pos; // position: first index
	GLint no; // number: second index
};


#define NUM_3VECS_PR_VERT 4



class ObjData
{
// Private Members:
	std::vector<Vec3f> vertPositions;
	std::vector<Vec3f> vertNormals;
	std::vector<Vec3f> vertTexture;
	std::vector<float> vertRadius;
	std::vector<Vec3f> vertColors;

	std::vector<std::vector<VertexDataPointers>> vertices; // lookuptable with sets of indices into the vertex tables grouped on positions
	std::vector<VertexIndex> faces; // lookuptable with sets of (three and three for triangles) indices into the vertices-lookuptable
	std::vector<std::vector<int>> neighbours;
	std::vector<std::vector<float>> distances;
	GLint* nodeOffsets; // the offsets of the group of vertices belonging to each node.
	GLint* nborCounts;

	GLint numVertices;
	GLint numFaces;
	GLint numVertsPrFace;
	GLint numMaxNbors;


// Private helper methods:
	void makeLookupTables(std::vector<RawFaceDataPointers>* rawFacesTable);
	void makeNeighbourMatrix(void);
	void makeDistMatrix(void);
	void makeNodeOffsetTable(void);
	std::vector<RawFaceDataPointers> loadDataFromObj(std::ifstream* instream, int maxLineLength, GLfloat scale); //,std::set<RawPos> uniquePosStrings, std::vector<int> posIdxTranslation		
	inline float vertDistance(int posIndex1, int posIndex2);
	void checkPositions(unsigned int maxPos, GLfloat errThresh);
	inline bool dupCheck(int posIndex1, int posIndex2, float thresh);
	void checkNumVertices(void);
	void checkForUnusedPositions(void);
	void printVertexData(int pos, int no);
	void printVertexIndices(int pos, int no);

	inline Vec3f getLinkDir(int nodeidx, int nboridx)
	{
		GLfloat dist = distances[nodeidx][nboridx];
		Vec3f result;
		result.x = (vertPositions[nodeidx].x - vertPositions[neighbours[nodeidx][nboridx]].x)/dist;
		result.y = (vertPositions[nodeidx].y - vertPositions[neighbours[nodeidx][nboridx]].y)/dist;
		result.z = (vertPositions[nodeidx].z - vertPositions[neighbours[nodeidx][nboridx]].z)/dist;
		return result;
	}


public:
	// Read data from obj file
	ObjData(const char* objFileName, GLfloat scale, GLboolean makeMesh);
	~ObjData(void);
	// Produce vertex matrices
	GLint getNumVertices(void);
	//GLfloat* getVertexdata(void); //deprecated - my app needs separate vertex buffers
	GLfloat* getVertPositions(void);
	GLfloat* getVertNormals(void);
	GLfloat* getVertTexcoords(void);
	GLfloat* getVertRadiuses(void);
	GLfloat* getVertColors(void);
	GLfloat* getVertDisplacements(void);
	// Produce face matrix
	GLint getNumFaces(void);
	GLint getNumVertsPrFace(void);
	GLint* getFaces(void);
	// Produce mesh
	GLint* getVertexNodeMap(void); //generates a mapping between nodes (unique positions) and the vertices at these nodes(/positions)
	GLint* getNeighbourMatrix(void); // generates a table with lists of neighbours for each node.
	GLfloat* getDistanceMatrix(void);
	GLfloat* getLinkdirMatrix(void); // generates a table with (normalized) direction vectors for each link
	GLint getMaxNbors(void);
	GLint* getNborCounts(void);
	GLint getNumNodes(void);
	GLfloat* getNodeFuels(void);

	//Debug function:
	void checkNormals(void); //checks normals of vertices at one position, to see if they differ.
};
