#include "StdAfx.h"
#include "RenderContext.h"

RenderContext::RenderContext(void)
{

	// Load the identity matrix into the model, view, and projection matrices.
    loadIdMatrix4x4f(m_projectionMatrix);
    loadIdMatrix4x4f(m_viewMatrix);
	setLightPos(10.0f, 10.0f, 2.0f);
	memset(&clearColor, 0, sizeof(Vec4f));
    glClearColor(clearColor.x, clearColor.y, clearColor.z, clearColor.w);


}

RenderContext::~RenderContext(void)
{
}

void RenderContext::reset(void)
{

	// Load the identity matrix into the model, view, and projection matrices.

    loadIdMatrix4x4f(m_projectionMatrix);
    loadIdMatrix4x4f(m_viewMatrix);
}


void RenderContext::setClearColor(GLfloat red, GLfloat green, GLfloat blue, GLfloat asomething)
{
	clearColor.x = red;
	clearColor.y = green;
	clearColor.z = blue;
	clearColor.w = asomething;
	glClearColor(clearColor.x, clearColor.y, clearColor.z, clearColor.w);
}

void RenderContext::setLightPos(GLfloat x, GLfloat y, GLfloat z)
{
	lightPosition[0] = x;
	lightPosition[1] = y;
	lightPosition[2] = z;
}


void RenderContext::perspective(GLfloat fov, GLfloat aspect, GLfloat nearz, GLfloat farz)
{
    GLfloat range;

    range = tan(fov * PI_OVER_360) * nearz; /* 0.00872664625 = PI/360 */
    memset(m_projectionMatrix, 0, sizeof(GLfloat) * 16);
    m_projectionMatrix[0] = (2 * nearz) / ((range * aspect) - (-range * aspect));
    m_projectionMatrix[5] = (2 * nearz) / (2 * range);
    m_projectionMatrix[10] = -(farz + nearz) / (farz - nearz);
    m_projectionMatrix[11] = -1;
    m_projectionMatrix[14] = -(2 * farz * nearz) / (farz - nearz);
}

