#version 140

precision highp float;

uniform mat4 ProjectionMatrix;
uniform mat4 ViewMatrix;
uniform mat4 ModelMatrix;

uniform mat3 NormalMatrix;

uniform vec3 LightPosition;

const float SpecularContribution = 0.3;
const float DiffuseContribution = 1.0 - SpecularContribution;

in vec3 in_Position;
in vec3 in_Normal;
//in vec3 in_Color;
in vec4 in_Glow;
in vec3 in_TexCoord;
in float in_Displ;

out vec4 ex_Glow;
//out vec3 ex_Color;
//out vec3 ex_Normal;
out float LightIntensity;
out vec2 ex_TexCoord;

void main()
{
	vec4 mcVertex = vec4((in_Position-(in_Displ*in_Normal)), 1.0);
	mat4 mvMatrix = ViewMatrix * ModelMatrix;
	mat4 mvpMatrix = ProjectionMatrix * mvMatrix;

	
	vec3 ecPosition = vec3(mvMatrix * mcVertex);
	vec3 tnorm = normalize(NormalMatrix * in_Normal);
	vec3 lightVec = normalize(LightPosition - ecPosition);
	vec3 reflectVec = reflect(-lightVec, tnorm);
	vec3 viewVec = normalize(-ecPosition);
	
	float diffuse = max(dot(lightVec, tnorm), 0.0);
	float spec = 0.0;
	
	if (diffuse > 0.0)
	{
		spec = max(dot(reflectVec, viewVec), 0.0);
		spec = pow(spec, 16.0);
	}
	
	LightIntensity = max(DiffuseContribution * diffuse + SpecularContribution * spec, in_Glow.a);
	
	gl_Position = mvpMatrix * mcVertex;
	ex_Glow = in_Glow;
	//ex_Color = in_Color;
	//ex_Normal = in_Normal;
	ex_TexCoord = in_TexCoord.st;
}
