
#pragma OPENCL EXTENSION cl_khr_byte_addressable_store : enable
#pragma OPENCL EXTENSION CL_KHR_gl_sharing : enable


// Constant parameters:
const float t_ignite = 500.0;
const float fireTemp = 1000.0;
const float fuelScaling = 0.0001;//0.00001;
const float speedScaling = 0.005;
const float airTemp = 1.0;
const float maxtemp = 10000.0;
const float maxfuel= 10000.0;
const float minfuel= -0.000001;
const float fuelthresh = 0.000001;
const float displScaling = 0.5;
const float spreadSpeed = 0.0005;

__kernel void updatetemps(__global float * temps, __global int * readIgn, 
	__global int * writeIgn, __global float * fuel, const unsigned int nborswidth, __global int * nbors, 
	__global float * distances, __global int * nborcounts, __global float * linkspeeds, const float deltaT)
{
	
    size_t tid = get_global_id(0);
	int numnbors = nborcounts[tid];
	float linkspeed = 0.0;
	
	// STEP 1: ADVANCE LINK FLAMES AND IGNITE SELF IF ANY BURNS DOWN
	
	for (int i = 0; i < numnbors; i++)//for each link
	{
		if (readIgn[nbors[tid*nborswidth + i]] > 0) // if the other node has ignited
		{
			if (distances[tid*nborswidth + i] >= 0.0) //if this link is not already burned
			{
				linkspeed = speedScaling * linkspeeds[tid*nborswidth + i];
				if (linkspeed < 0.0)
					linkspeed = 0.0; 
				distances[tid*nborswidth + i] -= ((linkspeed + spreadSpeed)  * deltaT);
				// if the link now has burned down, ignite self
				if (distances[tid*nborswidth + i] < 0.0)
				{
					writeIgn[tid] = 1;
				}
			}
		}
	}
	
	//STEP 2: INCREASE NODE TEMP AND DECREASE FUEL
		
	float heatProduced = 0.0;
	float heatLost = 0.0000001 * (temps[tid]-airTemp) * (temps[tid]-airTemp) * deltaT;
	
	if (readIgn[tid] > 0 && fuel[tid] > minfuel)
	{
		float adjustedTemp = temps[tid]+1.0;
		if (adjustedTemp <= 0.0)
			adjustedTemp = 0.1;
		heatProduced = fireTemp * adjustedTemp * deltaT;
		fuel[tid] -= (deltaT * fuelScaling);
	}
	temps[tid] += heatProduced-heatLost;
	

	if (temps[tid] > maxtemp)
	{
		temps[tid] = maxtemp;
	}
	if (temps[tid] < -maxtemp)
	{
		temps[tid] = -maxtemp;
	}
	if (fuel[tid] > maxfuel)
	{
		fuel[tid]=maxfuel;
	}
	if (fuel[tid] < minfuel)
	{
		fuel[tid] = minfuel;
	}
	
    
}

__kernel void updatecolors(__global float * readtemps, __global float * fuelinit, __global float * fuelrem, 
	__global int * mapping, __global float * colors, __global float * displ)
{
	
	size_t tid = get_global_id(0);

	float color[4];
	
	color[0] = 0.1; //red
	color[1] = 0.9; //green
	color[2] = 0.1; //blue
	color[3] = 0.0; //alpha
	
	if (fuelrem[tid] < fuelthresh)
	{
		color[0] = 0.0001*readtemps[tid];//0.1; 
		color[1] = 0.1; 
		color[2] = 0.1;
		color[3] = 0.9;
	} else if ((readtemps[tid] - t_ignite) > 0.1)
	{
		color[0] = 0.001*readtemps[tid]; 
		color[1] = 0.00001*readtemps[tid]; 
		color[2] = 0.1;
		color[3] = 0.0001*readtemps[tid];
	}
	
	//clamp to 0.0 and 1.0
	for (int i = 0; i < 4; i++)
	{
		if (color[i] < 0.0)
			color[i] = 0.0;
		if (color[i] > 1.0)
			color[i] = 1.0;
	}
	
	// update color and displacement buffers
	int vertidx = 0;
	
	if(tid > 0)
	{
		vertidx = mapping[tid-1];
	}
	for (; vertidx < mapping[tid]; vertidx++)
	{
		colors[4*vertidx+0] = color[0];
		colors[4*vertidx+1] = color[1];
		colors[4*vertidx+2] = color[2];
		colors[4*vertidx+3] = color[3];
		displ[vertidx] = (fuelinit[tid] - fuelrem[tid])*displScaling;
	}
	
}

__kernel void updatewind(__global float * linkdirs, const unsigned int nborswidth, 
	__global float * windvec, __global float * linkspeeds)
{
	size_t tid = get_global_id(0);
	for (int i = 0; i < nborswidth; i++)
	{
		float dirx = linkdirs[(tid*nborswidth+i)*3+0];
		float diry = linkdirs[(tid*nborswidth+i)*3+1];
		float dirz = linkdirs[(tid*nborswidth+i)*3+2];
		
		linkspeeds[tid*nborswidth+i] = (dirx*windvec[0])+(diry*windvec[1])+(dirz*windvec[2]);
	}

}
