__metaclass__ = type # Make sure we get new style classes
import re
import time

filename = r'jorgentree.obj'#r'jtreesimple.obj'#r'jtreesimpletest.obj'
newfilename = r'jorgentree_uniq.obj'
#filename = r'jorgenhires.obj'
#newfilename = r'jorgenhires_uniq.obj'

class PosList(list):
    def __init__(self, *args):
        super(PosList,self).__init__(*args)

    def appendUnique(self,pos):
        """
        searches the list for elements equal to 'pos',
        and, does one of two things:
        1) An equal element is found
            - the index of the existing elment is returned,
              and nothing is added to the list
        2) There are no element in the list equal to 'pos'
            - append 'pos' to the end of the list,
              and return something to indicate it is added,
              and preferrably also the index of the appended element
              (such as the negative of the index).
        New thougth: maybe I don't really need an indicator of whether or not 'pos' is found,
        as long as I return the correct index?
        """
        found = False
        index = 0
        #search list here, note the index of the found element
        try:
            index = self.index(pos) #super(PosList,self).index(pos)
            found = True
        except ValueError:
            found = False
        if(found): # cant simply use index, because zero (first index) interprets as False
            index = index # maybe I dont need to do anything in this block
        else:
            index = self.__len__() #super(PosList,self).__len__() dont need to call super's len...
            self.append(pos) #super(PosList,self).append(pos)
        return (index, found)


pat = re.compile(r"""
    ^\s*        # start with zero or more whitespaces (wsp)
    (\#.*|      # alt. 1 (comment): match '#' and zero or more characters (incl wsp)
    [a-z]{1,3}[0-9]*) # alt. 2: match 1-3 letters (like v,n,vt,vt1,vt2...)
    \s+         # follow any of these alternatives with one or more wsp...
    ((?:        # save outer, but not inner group
    -?[0-9/]+   # first part: a set of numbers and/or slashes possibly preceeded by a minus
    (?:\.[0-9]+)? # second part: possible fractional part
    (?:e-?[0-9]+)? # third part: possible exponential
    \s*         # the number groups may be suceeded by zero or more wsp
    ){0,5})     # reapeat inner (not-saved) group
    (?:$|\#)     # end with end of line or comment (dont save)
    """,re.VERBOSE | re.IGNORECASE)
subpat = re.compile(r'(-?[0-9/]+(?:\.[0-9]+)?(?:e-?[0-9]+)?)',re.I) #matches each numbers group
with open(filename,'r',-1) as fin: #open read-only, default buffersize
    print "Reading", filename, "..."
    lno = 0
    pos = PosList()
    norm = []
    tex = []
    transtable = [] 
    face = []
    tan = [] 
    rad = []
    for line in fin:
        lno+=1
        m = pat.match(line)
        if m:
            mtype = m.group(1)
            if mtype[0] == '#':
                #print "Found comment on line", lno
                pass
            elif mtype == 'v':
                #print "Found position on line", lno
                subm = subpat.findall(m.group(2))
                #print(subm)
                insidx = pos.appendUnique(subm)[0]
                transtable.append(insidx)
            elif mtype == 'vn':
                #print "Found normal on line", lno
                subm = subpat.findall(m.group(2))
                #print(subm)
                norm.append(subm)
            elif mtype == 'vt':
                #print "Found tex-coord on line", lno
                subm = subpat.findall(m.group(2))
                #print(subm)
                tex.append(subm)
            elif mtype == 'vt1':
                #print "Found tangent on line", lno
                subm = subpat.findall(m.group(2))
                #print(subm)
                tan.append(subm)
            elif mtype == 'vt2':
                #print "Found radius on line", lno
                subm = subpat.findall(m.group(2))
                # I believe Jorgen thresh the radius on 0.005 to avoid div by zero...
                #if subm[0] == '0.005':
                #    subm[0] = '0'                    
                #print(subm)
                rad.append(subm)
            elif mtype == 'f':
                #print "Found face on line", lno
                subm = subpat.findall(m.group(2))
                """
                subm is a list of sets of vertex-indices on the form POS/NORM/TEX
                where any of POS, NORM or TEX can be empty.
                f.ex.: ['2/5/2', '1/9/2', '3//2]
                these should be split up and stored as complete lists, like 
                f.ex.: [[2,5,2], [1,9,2], [3,0,2]]
                Indices start from 1, and zero indicates an empty index (there are no normals/textures)
                """
                newfelem = []
                subsubpat = re.compile(r'\s*([0-9]+)\s*/\s*([0-9]*)\s*/\s*([0-9]*)')
                for tri in subm:
                    m = subsubpat.match(tri)
                    newvelem = []
                    newvelem.append(m.group(1))
                    newvelem.append(m.group(2))
                    newvelem.append(m.group(3))
                    newfelem.append(newvelem)
                face.append(newfelem)
                #print(face[-1])
            else:
                print "  Found unknown data type: '", mtype, "' on line", lno
        else:
            print "  Empty line or unknown data on line", lno, ": '", line[0:-2], "'"
print "  Done reading file."

#create dummy lists for any missing vertex data (normals, tex-coords, etc.), and replace zero ('0') in 'face' with 1?



# update position index for each face according to transtable
# this must be done after we have collected the full transtable
print "Updating position indices in face-list..."
for f in face:
    for v in f:
        rawidx = int(v[0])
        try:
            temptest = transtable[rawidx-1]+1 # face list is 1-indexed while transtable is zero-idx
        except IndexError:
            print "  Index out of bounds:", rawidx, ", Length transtable is", len(transtable)
        v[0] = str(temptest)
print "  Done updating position indices."
     
#check for infinitesimal triangles (two (or three) positions are equal)
print "Checking for infinitesimal triangles..."
trino = 0
numremovedtri = 0
facelength = len(face)
while trino < facelength:
    if face[trino][0][0] != face[trino][1][0] and \
       face[trino][0][0] != face[trino][2][0] and \
       face[trino][1][0] != face[trino][2][0]:
        trino += 1 # do not increase trino when f is deleted!        
    else:
        #print "Warning: Found infinitesimal triangle on face number", trino, "- position indices:", face[trino][0][0], face[trino][1][0], face[trino][2][0]
        numremovedtri+=1
        del face[trino]
        facelength = len(face) #must update length of face
        #if trino < facelength:
        #    print "Test: new face on", trino, ":", face[trino]
print "  Done: Found and removed", numremovedtri, "infinitesimal triangles."


#remove unused positions
print "Removing unused positions..."
usecount = [False] * len(pos)
for f in face:
    for v in f:
        usecount[int(v[0])-1] = True
print "  Found", usecount.count(False), "unused positions."

transtable = range(len(pos))
unusedcount = 0
for v in range(len(transtable)):
    if not usecount[v]:
        try:
            del pos[transtable[v]]
        except IndexError:
            print "  ERROR: Index out of range:", transtable[v], "(was", v, "), length of pos:", len(pos)
        unusedcount += 1
        for rest in range(v,len(transtable)):
            transtable[rest] -= 1
print "  Done: Found and removed", unusedcount, "unused positions."


# update position index for each face according to transtable again
print "Updating position indices in face-list again..."
for f in face:
    for v in f:
        rawidx = int(v[0])
        try:
            temptest = transtable[rawidx-1]+1 # face list is 1-indexed while transtable is zero-idx
        except IndexError:
            print "  Index out of bounds:", rawidx, ", Length transtable is", len(transtable)
        v[0] = str(temptest)
print "  Done updating position indices."

# DEBUG: Checking usecount again:
print "Checking usecount again..."
usecount = [False] * len(pos)
for f in face:
    for v in f:
        usecount[int(v[0])-1] = True

print "  New unused count:", usecount.count(False), "(should be zero)"


#writeback to new file
with open(newfilename,'w',-1) as fout: #open write-only, default buffersize
    print "Writing corrected data to file", newfilename, "..."
    fout.write('#uniquified obj-file made from "')
    fout.write(filename + '" on ' + time.asctime() + '.\n')
    print "  Writing", len(pos), "positions..."
    fout.write('\n# ' + str(len(pos)) + ' position coordinates.\n')
    fout.writelines(['v ' + p[0] + ' ' + p[1] + ' ' + p[2] + '\n' for p in pos])
    print "  Writing", len(norm), "normals..."
    fout.write('\n# ' + str(len(norm)) + ' normals.\n')
    fout.writelines(['vn ' + n[0] + ' ' + n[1] + ' ' + n[2] + '\n' for n in norm])
    print "  Writing", len(tex), "texture coords..."
    fout.write('\n# ' + str(len(tex)) + ' texture coordinates (3rd coord is flip).\n')
    fout.writelines(['vt ' + t[0] + ' ' + t[1] + ' ' + t[2] + '\n' for t in tex])
    print "  Writing", len(tan), "tangents..."
    fout.write('\n# ' + str(len(tan)) + ' tangents.\n')
    fout.writelines(['va ' + a[0] + ' ' + a[1] + ' ' + a[2] + '\n' for a in tan])
    print "  Writing", len(rad), "radiuses (radii?)..."
    fout.write('\n# ' + str(len(rad)) + ' radiuses (radii?) (2nd coord is branch level).\n')
    fout.writelines(['vr ' + r[0] + ' ' + r[1] + ' ' + r[2] + '\n' for r in rad])
    print "  Writing", len(face), "faces..."
    fout.write('\n# ' + str(len(pos)) + ' faces.\n')
    for f in face:
        fout.write('f ')
        for v in f:
            fout.write(str(v[0]) + '/' + str(v[1]) + '/' + str(v[2]) + ' ')
        fout.write('\n')
    fout.write('\n# EOF\n\n')
print "  Done writing corrected data to file", newfilename

print "All done!"
            
    



